/*
ESUP-portail is a french academic project developed under the GPL (General Public License) augmented according to the following :
A binary or source file developped by ESUP-portail can be used or compiled with products under Apache license.
The official english text of the GPL can be found here : http://www.gnu.org/licenses/gpl.html .
A non official french translation can be found here : http://www.linux-France.org/article/these/gpl.html .
The different kinds of licenses governing the products developed by the Apache foundation can be found here : http://www.apache.org/licenses .
It follows that you can as well as use download contents as well modify and redistribute them provided you respect the GPL terms.
Downloading and using such contents do not provide any guaranty.
Be sure that you have well understood the terms of the license before using the contents it covers.
The ESUP-portail distribution includes the following distributions :
    * UPortal :
      software developed by JA-SIG (Java Architecture - Special Interest Group)
      You can find the license page here : http://mis105.udel.edu/ja-sig/uportal/license.html
    * CAS :
      SSO solution developed by Yale University
      You can find the project page here : http://www.yale.edu/tp/auth
    * Cocoon :
      XML framework distributed by the Apache foundation under Apache license;
      Please find the full text here : http://cocoon.apache.org/2.1/license.html
    * Mod_dav:
      A DAV module for Apache web server
      You can find the project page here : http://www.webdav.org/mod_dav
    * IMP :
      webmail from Horde application framework
      You can find the project page here : http://www.horde.org
    * . To be completed
*/
 
package org.esupportail.portal.channels.CPref;   
 

import java.util.Hashtable;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;

import org.apache.commons.logging.LogFactory;
import org.esupportail.portal.channels.CPref.LDAP.LDAPHandler;
import org.esupportail.portal.channels.CPref.LDAP.Exceptions.LDAPException;
import org.esupportail.portal.channels.CPref.config.Config;
import org.esupportail.portal.channels.CPref.plugins.AskPassword;
import org.esupportail.portal.utils.channels.FrameWorkException;
import org.esupportail.portal.utils.channels.MainChannel;
import org.esupportail.portal.utils.channels.SubChannel; 
import org.esupportail.portal.utils.channels.plugins.Message;
import org.esupportail.portal.utils.channels.plugins.MessageBean;
import org.jasig.portal.ChannelRuntimeData;
import org.jasig.portal.PortalException;
import org.jasig.portal.security.IPerson;


public abstract class SecureAction  extends SubChannel{ 


	protected final static String ETAT_VISUALISATION = "visu";
	protected final static String ETAT_MODIFICATION = "main";
	protected final static String ETAT_VALIDATION = "valid";

	protected String etat = ETAT_VISUALISATION;
	protected String dernierEtat = ETAT_VISUALISATION;
    /**
	 * @param mainChannel
	 */
	public SecureAction(MainChannel mainChannel) {
		super(mainChannel);
		logguer = LogFactory.getLog(CPref.class);
	}

	/* (non-Javadoc)
	 * @see org.esupportail.portal.utils.channels.ISubChannel#init(org.jasig.portal.ChannelRuntimeData)
	 */
	public Boolean init(ChannelRuntimeData rd) throws PortalException,
			FrameWorkException {
		Boolean ret = super.init(rd);
		xslParameters.put("toHttpsURL","HTTPSGateway?tohttps="+runtimeData.getBaseActionURL());
		xslParameters.put("backHttpURL","HTTPSGateway?backhttp="+runtimeData.getBaseActionURL());
		return ret;
	}
	
	/*nombre d'essai de mot de passe*/
	private final int nbTry = 3;
	
	//protected String password = "";
	
	//protected boolean identified = false;
	
	protected boolean nonSecure = false;
	
	protected boolean forbidden = false;
	
	/*nombre d'essai tents*/
	private int nbTrySession = nbTry;
	
	
	protected String dn = null;  
	
	public LDAPHandler LDAPconnexion = null;
	
/**********************************************/
/************FONCTIONS ABSTRAITES**************/

	/**
	 * fonction a effectuer une fois identifi
	 */
	public abstract Boolean identified() throws FrameWorkException;
	 
	

/**********************************************/
/************FONCTION DU MAG*******************/	
	/* (non-Javadoc)
	 * @see org.esupportail.portal.utils.channels.ISubChannel#setXML()
	 */
	public Boolean setXML() throws FrameWorkException{
		
		if (forbidden){
			logguer.error("SecureAction::setXml() : Action interdite");
			throw new FrameWorkException("Action interdite");
		}

		if (isIdentified()){
			identified();
			return Boolean.TRUE;
		}
		else{
			return Boolean.FALSE;
		}
		
			

	}
	

/**********************************************/
/***********FONCTIONS D'IDENTIFICATION*********/	
	/**
	 * Oriente la canal sur l'action qui convient en fonction
	 * de l'authentification
	 * 
	 * @throws FrameWorkException
	 */
	public  boolean isIdentified() throws FrameWorkException{
		
		
		/*
		 * Dans certain cas il ne faut pas demander de mot de passe : 
		 *  
		 */
		if (nonSecure){
			nbTrySession = nbTry;
			return true;
		}
		
		/*
		 * Dj authentifi
		 *  
		 */
		if ((mainChannel.getStaticData().getParameter("CPrefVarIdentified")!=null) 
			&& (mainChannel.getStaticData().getParameter("CPrefVarIdentified").equals("true"))
			&& (mainChannel.getCurrentAction().getParams().get("forceAskPassword")==null)/*Cas ou on doit redemander le mot de passe*/
			){			
			nbTrySession = nbTry;
			return true;
		}
		
		/*Cas ou une erreur de saisie s'est produite.*/
		if ( (etat.equals(ETAT_MODIFICATION) && (dernierEtat.equals(ETAT_VALIDATION)) )){
			nbTrySession = nbTry;
			return true;
		}
		
		/*
		 * Lancement de l'authentification
		 *  
		 */
		if (verifPassword((String)this.runtimeData.getParameter("actualPass"))){ 
		    if (logguer.isDebugEnabled()){
		        logguer.debug("SecureAction::isIdentified() : mot de passe vrifi");
		    }
		    initNbTry();
			mainChannel.getStaticData().setParameter("CPrefVarIdentified","true");
			return true;
		}
		else{
			
			if (nbTrySession>0){
				try {
					AskPassword.display(mainChannel,this.runtimeData,mainChannel.getCurrentAction().getName(),!(nbTrySession==nbTry));
				}
				catch (PortalException e) {
					throw new FrameWorkException("Erreur dans l'affichage du pluggin");
				}
				nbTrySession--;
				//mainChannel.getStaticData().setParameter("CPrefVarIdentified","false");
				deletePassword();
				return false;
			} 
			else{

				try {
					Message.message(mainChannel, runtimeData, new MessageBean("Nombre de tentatives puis.",Message.ERROR), "default",new Hashtable());
				}
				catch (PortalException e) {
					throw new FrameWorkException("Erreur dans l'affichage du pluggin");
				}
				initNbTry();
				//mainChannel.getStaticData().setParameter("CPrefVarIdentified","false");
				deletePassword();
				return false;
			}
		}    
	}
	
	protected void initNbTry(){
		nbTrySession = nbTry;
	}
	
	/**
	 * Vrifie le mot de passe saisi
	 * 
	 * @param pass
	 * @return
	 * @throws FrameWorkException
	 */
	public boolean verifPassword(String pass) throws FrameWorkException{
		
		if ( (pass==null) || (pass.equals("")) )return(false);
		
		setPassword(pass);
		
		this.openLDAP(true);
		
		boolean auth=false;

		try {						
		    auth = LDAPconnexion.authenticate();
        } catch (LDAPException e) {
        	setPassword("");
            logguer.error("SecureAction::verifPassword() : le mot de passe de "+(String)mainChannel.getStaticData().getPerson().getAttribute(IPerson.USERNAME).toString()+" n'est pas vrifi ");
		}
		
		
		if (auth){
		    if (logguer.isDebugEnabled()){
		        logguer.debug("SecureAction::verifPassword() : mot de passe vrifi");
		    }
		}

		this.closeLDAP();
		return auth;
	}
	
	protected void setPassword(String a){
		mainChannel.getStaticData().setParameter("CPrefVarPassword",a);
	}
	
/**********************************************/
/*********FONCTIONS DE CONNEXION LDAP**********/	
	
	/**
	 * Ouvre une connexion anonyme LDAP
	 * 
	 */
	public void openLDAP(){
		openLDAP(false);
	}
	/**
	 * Ouvre une connexion LDAP
	 * 
	 * @param secure : true pour une connexion non anonyme, false sinon
	 */
	public void openLDAP(boolean secure){
		
		if (LDAPconnexion==null)
			LDAPconnexion = new LDAPHandler();

		if (secure){ 
			findDn();
			LDAPconnexion.setBindDn(dn);
			LDAPconnexion.setBindPass(mainChannel.getStaticData().getParameter("CPrefVarPassword"));
			LDAPconnexion.setUrl(Config.getInstance().getLdapWriteUrl());
		}
		else{ 
			LDAPconnexion.setBindDn(Config.getInstance().getBindDn());
			LDAPconnexion.setBindPass(Config.getInstance().getPassDn());
			LDAPconnexion.setUrl(Config.getInstance().getLdapReadUrl());
		}
		
		LDAPconnexion.setMaxEntries(1);
		LDAPconnexion.setBase(Config.getInstance().getLdapBase());
		LDAPconnexion.setFilter(Config.getInstance().getFilterDn((String)mainChannel.getStaticData().getPerson().getAttribute(IPerson.USERNAME).toString()));
		LDAPconnexion.setScope(SearchControls.SUBTREE_SCOPE);
	}
	
	public void closeLDAP(){
		if (LDAPconnexion!=null){
			LDAPconnexion.close();
		}
	}

	private boolean findDn(){
	    
	    if (dn==null){
	        LDAPHandler LDAP = new LDAPHandler();
	        String[] att = new String[0];
	        LDAP.setAttributes(att);
	        LDAP.setMaxEntries(0);
			LDAP.setBindDn(Config.getInstance().getBindDn());
			LDAP.setBindPass(Config.getInstance().getPassDn());
	        LDAP.setUrl(Config.getInstance().getLdapReadUrl());
	        LDAP.setBase(Config.getInstance().getLdapBase());
	        LDAP.setFilter(Config.getInstance().getFilterDn((String)mainChannel.getStaticData().getPerson().getAttribute(IPerson.USERNAME).toString()));
	        LDAP.setScope(SearchControls.SUBTREE_SCOPE);
			
			NamingEnumeration answer = null; 
			
			try {
			    answer = LDAP.search();
            } catch (LDAPException e) {
                logguer.error("SecureAction::findDn() : une erreur est survenue lors de la recherche du DN de "
                        +mainChannel.getStaticData().getPerson().getAttribute(IPerson.USERNAME).toString());
                dn = mainChannel.getStaticData().getPerson().getAttribute(IPerson.USERNAME).toString();
                return false;
            }
			
            int nbResult = 0;
            
            try {				
    			/*Parcours des rsultats obtenus*/                
    			while (answer.hasMoreElements()) {
    				SearchResult sr;
    				sr = (SearchResult) answer.next();			
    				 dn = sr.getName();	
 				    nbResult ++;
    			}
    		} catch (NamingException e) {
    		    logguer.error("SecureAction:findDn() : NamingException ",e);
    		}
    		finally{
    		    LDAP.close();
    		}
    		
    		if (nbResult!=1){
    		    logguer.error("SecureAction:findDn() : "+nbResult+" rsultat(s) trouv(s) => ABANDON");
    		    dn = mainChannel.getStaticData().getPerson().getAttribute(IPerson.USERNAME).toString();
    		    return false;
    		}
    		
    		dn += ","+Config.getInstance().getLdapBase();

		    if (logguer.isDebugEnabled()){
		        logguer.debug("SecureAction:findDn() : dn calcul : "+dn);
		    }
	    }
	    
	    

	    
	    return true;
	}
/**********************************************/
/************FONCTIONS DE SETTER **************/	
		
	public void setNonSecure(boolean isNonSecure){
		nonSecure = isNonSecure;
	}

	public void setIdentidied(boolean isIdentified){
		mainChannel.getStaticData().setParameter("CPrefVarIdentified",Boolean.toString(isIdentified));
	}
	
	public void setForbidden(boolean isForbidden){
		forbidden = isForbidden;
	}

	public void deletePassword(){
		if (!Config.getInstance().isPasswordInSession()){
			setPassword("");
			mainChannel.getStaticData().setParameter("CPrefVarIdentified","false");
		}
	}
}
