/*
ESUP-portail is a french academic project developed under the GPL (General Public License) augmented according to the following :
A binary or source file developped by ESUP-portail can be used or compiled with products under Apache license.
The official english text of the GPL can be found here : http://www.gnu.org/licenses/gpl.html .
A non official french translation can be found here : http://www.linux-France.org/article/these/gpl.html .
The different kinds of licenses governing the products developed by the Apache foundation can be found here : http://www.apache.org/licenses .
It follows that you can as well as use download contents as well modify and redistribute them provided you respect the GPL terms.
Downloading and using such contents do not provide any guaranty.
Be sure that you have well understood the terms of the license before using the contents it covers.
The ESUP-portail distribution includes the following distributions :
    * UPortal :
      software developed by JA-SIG (Java Architecture - Special Interest Group)
      You can find the license page here : http://mis105.udel.edu/ja-sig/uportal/license.html
    * CAS :
      SSO solution developed by Yale University
      You can find the project page here : http://www.yale.edu/tp/auth
    * Cocoon :
      XML framework distributed by the Apache foundation under Apache license;
      Please find the full text here : http://cocoon.apache.org/2.1/license.html
    * Mod_dav:
      A DAV module for Apache web server
      You can find the project page here : http://www.webdav.org/mod_dav
    * IMP :
      webmail from Horde application framework
      You can find the project page here : http://www.horde.org
    * . To be completed
*/
 
package org.esupportail.portal.channels.CPref;   
 
  
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.SearchResult;

import org.esupportail.portal.channels.CPref.LDAP.Exceptions.LDAPException;
import org.esupportail.portal.channels.CPref.config.Config;
import org.esupportail.portal.channels.CPref.config.LdapAttribut;
import org.esupportail.portal.channels.CPref.formats.Format;
import org.esupportail.portal.utils.channels.ActionParam;
import org.esupportail.portal.utils.channels.FrameWorkException;
import org.esupportail.portal.utils.channels.MainChannel;
import org.esupportail.portal.utils.channels.plugins.Message;
import org.esupportail.portal.utils.channels.plugins.MessageBean;
import org.esupportail.portal.utils.channels.plugins.RestrictedAction;
import org.jasig.portal.ChannelRuntimeData;
import org.jasig.portal.PortalException;
import org.jasig.portal.security.IPerson;


public abstract class UpdateAction  extends SecureAction{ 
	
	
    /**
	 * @param mainChannel
	 */
	public UpdateAction(MainChannel mainChannel) {
		super(mainChannel);
	}
	 
	
	protected Vector ldapFieldsToModify = new Vector();
	protected Vector ldapFieldsToDisplay = new Vector();
	protected Hashtable ldapFieldsToXsl = new Hashtable();
	/*Champ sui contient le mot de passe dans le LDAP ( modifier dans LDAPHandler)*/
	private final static String LDAP_USER_PASSWORD = "userPassword";
	
	/*Mthode  redfinir dans les sous classes, si ncessaire*/
	protected boolean prepareSpecifique() throws FrameWorkException {
		return true;
	}
	
	private void setEtat(String e){
		dernierEtat = etat;
		etat = e;
		/*modification de l'affichage*/
		mainChannel.getCurrentAction().setSsltitle(etat);
	}
	/**
	 * Valeurs cls de l'action
	 */	
	private boolean prepare() throws FrameWorkException {
			
		if (runtimeData.getParameter("etat")!=null){
			/*changement d'tat*/
			setEtat(runtimeData.getParameter("etat"));
		}
		else if (mainChannel.getPreviousAction().getName().equals(mainChannel.getDefaultAction())){
			/*arrivee depuis le menu*/
			setEtat(ETAT_VISUALISATION);
		}
		
		prepareListFieldParam();
		prepareListFieldRead();
		prepareListFieldWrite();
		prepareSpecifique();
		return true;
	}
	
	/** 
	 * Postionnement des champs en paramtres
	 */
	public void prepareListFieldParam() throws FrameWorkException {
		ldapFieldsToXsl = new Hashtable();
		ActionParam param = (ActionParam)mainChannel.getCurrentAction().getParams().get("attributeXsl");
		if (param!=null){
			List atts = (List)param.getValues();
			
			for (int i=0;i<atts.size();i++){
				String[] tmp = new String[1];
				ldapFieldsToXsl.put((String)atts.get(i),tmp);
			}
		}
	}
	
	/**
	 * Spcification des champs LDAP  utiliser en lecture
	 * Ceux spcifis en write sont pris en compte aussi dans les read
	 */	
	protected void prepareListFieldRead() throws FrameWorkException {
		ldapFieldsToDisplay.clear();
		ActionParam param = (ActionParam)mainChannel.getCurrentAction().getParams().get("attributeRead");
		if (param!=null){
			List atts = (List)param.getValues();
			for (int i=0;i<atts.size();i++){				
				//ldapFieldsToDisplay.add(Config.getInstance().getAttribLdap((String)atts.get(i)));
				ldapFieldsToDisplay.add((String)atts.get(i));
			}
		}
	}
	
	/**
	 * Spcification des champs LDAP  utiliser en criture
	 */	
	protected void prepareListFieldWrite() throws FrameWorkException {
		ldapFieldsToModify.clear();
		ActionParam param = (ActionParam)mainChannel.getCurrentAction().getParams().get("attributeWrite");		
		if (param!=null){
			List atts = (List)param.getValues();
			for (int i=0;i<atts.size();i++){
				//ldapFieldsToModify.add(Config.getInstance().getAttribLdap((String)atts.get(i)));
				ldapFieldsToModify.add((String)atts.get(i));
			}
		}
		
	}	
	
/**********************************************/
/******FONCTIONS IMPLEMENTEES PAR DEFAUT*******/
	/**
	 * Mthode effectue une fois authentifi
	 * @return Boolean : 
	 * 	* Boolean.TRUE: validation effectue avec succs
	 * 	* Boolean.FALSE: redirection sur le plugin message
	 */
	public Boolean identified() throws FrameWorkException{
		try {
			return visualisation();
		} catch (FrameWorkException e) {
			try {
				Message.message(mainChannel,runtimeData,new MessageBean(e.getDisplayMessage(),Message.ERROR),"default",new Hashtable());
			} catch (PortalException e1) {
				logguer.error("PortalException : "+e1);
			}
			return Boolean.FALSE;
		} catch (PortalException e) {
			try {
				Message.message(mainChannel,runtimeData,new MessageBean(e.getMessage(),Message.ERROR),"default",new Hashtable());
			} catch (PortalException e1) {
				logguer.error("PortalException : "+e1);
			}
			return Boolean.FALSE;
		}
	}	

	/**
	 * Mthode permettant d'enregistrer les donnes
	 */
	public Boolean validation() throws FrameWorkException, PortalException {
		
		/*Ouverture de la connextion LDAP*/ 
		this.openLDAP(true);   
		LDAPconnexion.setAttributes(ldapFieldsToModify);
		
		/*Positionnement des valeurs  enregistrer*/
		Hashtable attribsValue = new Hashtable();	
		for (int i=0; i<ldapFieldsToModify.size();i++){
			String paramName = (String)getModifiedFileds().get(i);
			if (runtimeData.getParameterValues(paramName)!=null){
				attribsValue.put(paramName, (String[])runtimeData.getParameterValues(paramName)) ;	
			}
			else{
				attribsValue.put(paramName, new String[0]) ;	
			}
		}  
		LDAPconnexion.setAttributesValues(attribsValue);
		
		
		/*criture*/
		try {
			LDAPconnexion.write(dn);
		} catch (LDAPException e) {
			try {
				Message.message(mainChannel,runtimeData,new MessageBean(e.getDisplayMessage()),"default",new Hashtable());
			} catch (PortalException e1) {
				logguer.error("PortalException : "+e1);
			}
			return Boolean.FALSE;
		}
		
		/*Fermeture de la connextion LDAP*/ 
		this.closeLDAP();
		
		return Boolean.TRUE;
	}

/**********************************************/
/************FONCTION DU MAG*******************/	
	public Boolean init(ChannelRuntimeData rd) throws PortalException,FrameWorkException {
		super.init(rd);

		
		/* 
		 * Si l'utilisateur clique sur le bouton "Cancel" alors
		 * le MAG est redirig sur l'action par dfault
		 */
		String cancel = runtimeData.getParameter("Cancel");
		if (cancel!=null){
			mainChannel.redirect(rd, "default");
			initNbTry();
			return Boolean.FALSE;
		}
		 
		if (logguer.isDebugEnabled()){
		    logguer.debug("UpdataAction:init() : "+(String)mainChannel.getStaticData().getPerson().getAttribute(IPerson.USERNAME)+" essaye de se connecter  l'action "+mainChannel.getCurrentAction().getName());
		}
		
		/*
		 * Vrification de l'autorisation d'accs
		 */
		if (!RestrictedAction.filterAccess(mainChannel,mainChannel.getCurrentAction().getName(),Config.getInstance().getGroupmapping())){
		    logguer.error("UpdataAction:init() : "+(String)mainChannel.getStaticData().getPerson().getAttribute(IPerson.USERNAME)+" n'est pas autoris  se connecter  l'action "+mainChannel.getCurrentAction().getName());
			Message.message(mainChannel, runtimeData, new MessageBean("Vous n'tes pas autoris  accder  cette page."),"default",new Hashtable());
			return Boolean.FALSE;
		}
		
		
		
		
		/*
		 * Appel de la mthode prpare
		 */
		if (!prepare())
			return Boolean.FALSE;
		
		
		/*
		 * Si l'action de validation est appelle directement sans passer par
		 * le formulaire de modification
		 */	
		if ((etat.equals(ETAT_VALIDATION))
				&& (!dernierEtat.equals(ETAT_MODIFICATION))
				){
			setForbidden(true);
			setIdentidied(false);
			return Boolean.TRUE;
		}
		

		
		/*
		 * Enregistrement de la saisie
		 *  if (mainChannel.getCurrentAction().getName().equals(actionValid)){
		 */
		if (etat.equals(ETAT_VALIDATION)){
			try {
				
				boolean validForm = true;
				
				try {
					validForm = verifAttribLDAP(ldapFieldsToModify); 
				} catch (FrameWorkException e1) {
					try {
						Message.message(mainChannel,runtimeData,new MessageBean(e1.getDisplayMessage()),"default",new Hashtable());
					} catch (PortalException e) {
						logguer.error("UpdataAction:init() : PortalException "+e);
					}
					return Boolean.FALSE;
				}
				
				if (!validForm){
					setEtat(ETAT_MODIFICATION);
				}
				else{
				
					Boolean res = validation();
					setNonSecure(true);
					deletePassword();
					return res;
				}
			} catch (PortalException e) {
				throw new FrameWorkException("Erreur dans la validation");
			}
			
		}

		
		/*
		 * Si l'action de modification est appelle apres la validation c'est
		 * qu'une erreur de saisie s'est glisse dans la saisie
		 * Il ne faut donc pas effacer le mot de passe
		 * 			
		*/	
		 
		if ( (etat.equals(ETAT_MODIFICATION))
				&&( (dernierEtat.equals(ETAT_VALIDATION)) || (dernierEtat.equals(ETAT_MODIFICATION)) )
			){
			return Boolean.TRUE;
		}	
		
		deletePassword();
		setNonSecure(false);
		setForbidden(false);
		if (etat.equals(ETAT_VISUALISATION)){
			setNonSecure(true);
		}
		/*if (subAction.equals(actionVisu)){
			setNonSecure(true);
		}*/
		return Boolean.TRUE;
	}
	
	
	
/**********************************************/
/************FONCTION D'ACTION*****************/
	/**
	 * Fonction qui rcupre les information du LDAP
	 * essentiellement pour la visualisation
	 */	
	public Boolean visualisation() throws FrameWorkException,PortalException{
		
		/*Concatnation des attributs sans l'attribut userPassword*/
		Vector listAttributesLdap = new Vector();
		listAttributesLdap.addAll(ldapFieldsToDisplay);
		listAttributesLdap.addAll(ldapFieldsToModify);	
		listAttributesLdap.removeAll(ldapFieldsToXsl.keySet());	
		listAttributesLdap.addAll(ldapFieldsToXsl.keySet());		
		
		boolean passwordARecuperer = false;
		if (listAttributesLdap.contains(LDAP_USER_PASSWORD)){
			listAttributesLdap.remove(LDAP_USER_PASSWORD);
			passwordARecuperer=true;
			
		}
		
		/*Ouverture d'une connexion*/
		this.openLDAP();
		
		/*Positionnement des attributs  rcuprer*/
		LDAPconnexion.setAttributes(listAttributesLdap);	
		if (passwordARecuperer){
			listAttributesLdap.add(LDAP_USER_PASSWORD);
		}
		
		NamingEnumeration answer = null; 
		try {
			answer = LDAPconnexion.search();
		} catch (LDAPException e1) {
			logguer.error("UpdateAction:visualisation() : LDAPException : ",e1);
			throw new FrameWorkException(e1.getDisplayMessage());			
		}
		
		boolean isError = false;
		StringBuffer xmlBuffer = new StringBuffer();  
		try {				
			/*Parcours des rsultats obtenus et cration de la chaine XML*/
			while (answer.hasMoreElements()) {
				int errorForm = 0;
				SearchResult sr;
				sr = (SearchResult) answer.next();					
				xmlBuffer.append("<INFOS>")  ;
				
				Iterator i = listAttributesLdap.iterator();
				while (i.hasNext()){			    
					String nomAttrib = (String)i.next();
					if (nomAttrib!=null){
						xmlBuffer.append("<ATTRIBUTE name='"+nomAttrib+"' ");
						Format format = getClassFormat(nomAttrib);
						
						
						/*Traitement des erreurs de saisie*/	
						if (runtimeData.getParameter("ERROR_"+nomAttrib)!=null){
							xmlBuffer.append("inError=\"1\" ");
							errorForm++;
						}
						xmlBuffer.append(">");
						
						/*n'affiche pas la valeur dans le xml mais a test les erreurs*/
						if (!nomAttrib.equals(LDAP_USER_PASSWORD)){
							/*Lors de l'ajout d'une valeur, on rcupre ce qu'avait saisi l'utilisateur*/
							if (runtimeData.getParameterValues(nomAttrib)!=null){
								String[] valAttrib = (String[])runtimeData.getParameterValues(nomAttrib);		
								for (int j=0;j<valAttrib.length;j++){
									xmlBuffer.append("<VALUE value=\""+valAttrib[j]+"\" />");											
								}
								/*Traitement des ajout de valeur*/	
								if (runtimeData.getParameter("addValue"+nomAttrib)!=null){
									xmlBuffer.append("<VALUE value=\"\" />");
								}
							}
							else if (sr.getAttributes().get(nomAttrib)!=null){
								
								try{			
	
									if (sr.getAttributes().get(nomAttrib)!=null){
										int k=sr.getAttributes().get(nomAttrib).size();
										/*Rcupration des valeurs de l'attribut dans le LDAP pour
										 * former la chaine xml*/
										String[] valAttrib = new String[k];		
										for (int j=0;j<k;j++){
											valAttrib[j]= (String)sr.getAttributes().get(nomAttrib).get(j);
											if (format!=null)
												valAttrib[j] = format.display(valAttrib[j]);
											valAttrib[j]= valAttrib[j].replaceAll("\"","&quot;");
											xmlBuffer.append("<VALUE value=\""+valAttrib[j]+"\" />");							
										}
										/*rcupration des valeurs  envoyer  la feuille xsl*/
										if (ldapFieldsToXsl.get(nomAttrib)!=null)
											ldapFieldsToXsl.put(nomAttrib,valAttrib);				
									}	
								}
								catch (Exception e){
									isError = true;
									logguer.error("UpdateAction:visualisation() : le champs '"+nomAttrib
											+"' n'est pas une String dans LDAP. Vous ne pouvez pas l'afficher dans ce canal.",e);
								}					
							}
						}
						
						
						if (format!=null){
							String[] ex = format.exemple();
							for (int j=0;j<ex.length;j++){
								ex[j]= ex[j].replaceAll("\"","&quot;");
								xmlBuffer.append("<EXEMPLE value=\""+ex[j]+"\" />");
							}
						}
						xmlBuffer.append("</ATTRIBUTE>");
					}
				}
				xmlBuffer.append("</INFOS>")  ;
				
				
				this.getXSLParameter().put("error",Integer.toString(errorForm));
				
			}
			
			if (isError){
				xmlBuffer.setLength(0);
				xmlBuffer.append("<ERROR typeError=\"pbWhileGetInfosFromLdap\"/><INFOS />");
			}
			
			/*passage des paramtres dans le xml pour la feuille xsl*/
			xmlBuffer.append(setParamFileds(ldapFieldsToXsl));
			
			this.xml = xmlBuffer.toString();
		} catch (NamingException e) {
			logguer.error("UpdateAction:visualisation() : NamingException "+e);
			return Boolean.FALSE;
		}
		finally{
			this.closeLDAP();
		}
		
		return Boolean.TRUE;
	}
	
	/**
	 * Rcupre le format pour un attribut LDAP
	 * @param attrib nom de l'attribut
	 * @return le format correspondant
	 * @throws FrameWorkException
	 * */
	private Format getClassFormat(String attrib) throws FrameWorkException{
		return getClassFormat(Config.getInstance().getAttribLdap(attrib));
	}
	
	/**
	 * Rcupre le format pour un attribut LDAP
	 * @param attrib un LdapAttribut
	 * @return le format correspondant
	 * @throws FrameWorkException
	 */
	private Format getClassFormat(LdapAttribut attrib) throws FrameWorkException{
		
		if (attrib==null)return null;
		
		String group = (String)mainChannel.getStaticData().get("plug.esup.restrictedAction");
		Class c;
		Format format = null;
		try {
			if (attrib.getFormat(group)!=null){
				c = Class.forName(attrib.getFormat(group));
				format = (Format)c.newInstance();				
			}			
		} catch (ClassNotFoundException e) {
			logguer.error("UpdateAction::verifAttribLDAP: ClassNotFoundException : "+attrib.getFormat(group));
			throw new FrameWorkException("Impossible de trouver le format : "+attrib.getFormat(group));
		} catch (InstantiationException e) {
			logguer.error("UpdateAction::verifAttribLDAP: InstantiationException : "+attrib.getFormat(group));
			throw new FrameWorkException("Impossible d'instancier le format : "+attrib.getFormat(group));
		} catch (IllegalAccessException e) { 
			logguer.error("UpdateAction::verifAttribLDAP: IllegalAccessException : "+attrib.getFormat(group));
			throw new FrameWorkException("IllegalAccessException : "+attrib.getFormat(group));
		}
		return format;
	}

/**********************************************/
/****FONCTIONS DE VERIFICATION DE SAISIE*******/	
	/**
	 * Valide la saisie des atributs en fonction du format dfini
	 * 
	 * @param listAttrib : liste des champs  vrifier 
	 * @return true s'il n'y a pas d'erreur de saisie
	 * @throws FrameWorkException
	 */
	public boolean verifAttribLDAP(Vector listAttrib) throws FrameWorkException{
		boolean valid = true;
		
		if ( (listAttrib==null) || (listAttrib.size()==0) ){
			logguer.error("UpdateAction::verifAttribLDAP: FrameWorkException : validation d'un formulaire dont les champs ne sont pas spcifis.");
			throw new FrameWorkException("Vous n'avez prcis aucun champ  valider.");
		}
		
		for (int i=0; i<listAttrib.size();i++){
			LdapAttribut attrib = (LdapAttribut)Config.getInstance().getAttribLdap((String)listAttrib.get(i));

			if (attrib!=null){
				Format format = getClassFormat(attrib);
				if (runtimeData.getParameterValues(attrib.getLdapName())!=null){
					String[] valueAttrib = runtimeData.getParameterValues(attrib.getLdapName());	
					
					
					if ( (format!=null) && (!format.check(valueAttrib)) ){
						this.getRuntimeData().setParameter("ERROR_"+attrib.getLdapName(),"1");
						valid = false;
						if (logguer.isDebugEnabled()){
						    logguer.debug("UpdateAction::verifAttribLDAP: "
						            +attrib.getLdapName()
						            +" : format \""
						            +attrib.getFormat((String)mainChannel.getStaticData().get("plug.esup.restrictedAction"))
						            +"\" non valide");
						}
													
					}
					else{
						/*Transformation selon le format*/
						if (format!=null)valueAttrib = format.transform(valueAttrib);
						runtimeData.setParameterValues(attrib.getLdapName(),valueAttrib);
					}					
				}
			}
		}
		
		return valid;
	}

		


/************************************************/	
/*****GETTER ET SETTER DES CHAMPS UTILISES*******/
	/**
	 * Positionne les champs LDAP  modifier.
	 * Tableau de String contenant les nom logiques des attributs 
	 * LDAP utiliss
	 * 
	 * @param ldapField tableau de champs LDAP qui seront modifis sur cette action
	 */
	public void setModifiedFileds(Vector fields) {
		if (fields==null)ldapFieldsToModify.clear();
		else ldapFieldsToModify = fields;
	}
	
	/**
	 * Permet d'obtenir la liste des noms logiques des champs en modification
	 * 
	 * @return String[] : liste des noms logiques des champs
	 */
	public Vector getModifiedFileds() {
		if (ldapFieldsToModify==null)ldapFieldsToModify=new Vector();
		return ldapFieldsToModify;
	}
		
	
	
	private StringBuffer setParamFileds(Hashtable fields) throws FrameWorkException {
		
		
		if (fields==null)return new StringBuffer();

		if (fields.size()==0)return new StringBuffer();
		
		StringBuffer xml = new StringBuffer();		
		
		xml.append("<PARAMS>");
		
		LDAPconnexion.setAttributes((String[])fields.keySet().toArray(new String[fields.size()]));
				
		String nomAttrib=null;
		Iterator i=fields.keySet().iterator();
		while (i.hasNext()){
			nomAttrib=(String)i.next();
			String[] param = (String[])fields.get(nomAttrib);
			if (param!=null){
				xml.append("<ATTRIBUTE name='"+nomAttrib+"' >");
				for (int j=0;j<param.length;j++){
					xml.append("<VALUE value=\""+param[j]+"\" />");
				}
				xml.append("</ATTRIBUTE>");
			}
		}
				
		xml.append("</PARAMS>");
		
		return xml;
	}

}
